#version 330

uniform sampler2D videoTexture;

uniform float aspectF;
uniform float offsetRot;
uniform float offsetScale;
uniform float offsetShearX;
uniform float offsetShearY;

uniform float offsetDistort;
uniform float uniformDistort;
uniform vec2 resV; // for distort

uniform vec2 offsetPos;
uniform vec2 normaliseExp;
uniform int mode;
uniform int iterationsL;

// debugging uniforms
//uniform float debug_left;
//uniform float debug_top;

out vec4 FragColour;
in vec2 out_uvs;

vec2 distortUV(vec2 uv, float k, float kcube)
{
    k /= 1000.0;
    k *= min(resV.x, resV.y); // normalise based on res

    vec2 t = uv - .50;
    float r2 = (t.x * t.x / aspectF) + (t.y * t.y * aspectF);
    float f = 0.;

    f = 1. + r2 * k;
    
    vec2 nUv = f * t;

    nUv += .50;
    
    return nUv;
}

vec2 modifyCoords(float posX, float posY, float rot, float scale, float shearX, float shearY, float distort)
{
    // rotation
    float sin_factor = sin(rot);
    float cos_factor = cos(rot);
    vec2 rotCoords = out_uvs;
    
    vec2 aspectRatio = vec2(aspectF, 1.0);
    
    ///*
    rotCoords -= 0.5;
    rotCoords /= aspectRatio;
    rotCoords *= mat2(cos_factor, sin_factor, -sin_factor, cos_factor);
    rotCoords *= aspectRatio;
    rotCoords += 0.5;
    //*/
    
    // distort
    vec2 uvmod = rotCoords;
    vec2 distortCoords = distortUV(uvmod, distort, 0.0);

    vec2 scaleCoords = vec2(scale) * (distortCoords - vec2(0.5));
    scaleCoords += vec2(0.5);
    
    // skew
    // for x, distance from 0.5 y (middle of the buffer, which is the anchor) multiplied by the shear amount
    float dist = (scaleCoords.y - 0.5) / normaliseExp.y;
    dist *= shearX * normaliseExp.x;
    scaleCoords.x += dist;
    
    dist = (scaleCoords.x - 0.5) / normaliseExp.x;
    dist *= shearY * normaliseExp.y;
    scaleCoords.y += dist;
    
    // position
    vec2 posV = vec2(posX, posY);
    vec2 posCoords = scaleCoords + posV;

    return posCoords;
}

void main( void )
{
    vec2 offsetCoordsA;
    vec2 offsetCoordsB;
    
    vec4 fragA = vec4(0.0);
    vec4 fragB = vec4(0.0);
    vec4 fragC = vec4(0.0);
    
    // test: iterations
    
    float iterationsL = float(iterationsL);
    float iterationMultF = 1.0 / iterationsL;
    vec2 offsetPosIncrV = iterationMultF * offsetPos;
    float offsetRotIncrF = iterationMultF * offsetRot;
    float offsetScaleIncrF = (offsetScale - 1.0) / iterationsL; // scale isn't additive (% based, 100% scale is fed as 1.0) so find the difference and divide it by the iterations and then it can be used as additive for each iteration
    float offsetDistortIncrF = iterationMultF * offsetDistort;
    
    float offsetShearIncrXF = iterationMultF * offsetShearX;
    float offsetShearIncrYF = iterationMultF * offsetShearY;
    
    // how far do we want to apply the samples to? If you tween from distortion of 0 to -2.0 (max), the resulting composite is so blurry that you can't actually tell what's going on. Therefore if the range it's applied to goes from 0.9 to 1.0, the actual shape details are vivid
    float uniformDistortStart = uniformDistort * 0.9;
    float offsetDistortUniformIncrF = iterationMultF * (uniformDistort - uniformDistortStart);
    
    // if scale is 200% and 10 iterations:
    // (offsetScale - 1.0) / 10.0
    // (2.0 - 1.0) / 10.0
    // 1.0 / 10.0 = 0.1
    // increments as 1.0, 1.1, 1.2, 1.3
    // but instead it should look like 1.01, 1.05, 1.1, etc
    
    // the question becomes if i have an end value of 10 and a start value of 1, and 5 iterations, what should i multiply by each time?
    // nth sqrt of(10 - 1)
    float scaleRootMultiplierF = pow(offsetScale, 1.0 / iterationsL);
    float scaleCurF = scaleRootMultiplierF;
    
    for(float i = 1; i <= iterationsL; i++)
    {
        offsetCoordsA     = modifyCoords(offsetPosIncrV.x * i,
                                         offsetPosIncrV.y * i,
                                         offsetRotIncrF * i,
                                         scaleCurF,
                                         offsetShearIncrXF * i,
                                         offsetShearIncrYF * i,
                                         (uniformDistortStart + (offsetDistortUniformIncrF * i)) + (offsetDistortIncrF * i));
        
        offsetCoordsB     = modifyCoords(-offsetPosIncrV.x * i,
                                         -offsetPosIncrV.y * i,
                                         -offsetRotIncrF * i,
                                         1.0 / scaleCurF,
                                         offsetShearIncrXF * i * 2.0,
                                         offsetShearIncrYF * i * 2.0,
                                         (uniformDistortStart + (offsetDistortUniformIncrF * i)) + (offsetDistortIncrF * i * 2.0));
        
        
        // notes on distort: getting different colour results depending if it's positive or negative distort, disregard the following because i haven't worked it out:
        // it was this meaning that both aberrated channels would be distorted the same way, with one recieving twice as much as the other. however this was confusing because the R&B result didn't give those colours in the fringe, you had to set it to G&B. The following gives the correct colour results, it sends one channel in +distortF and the other in -distortF
        //(uniformDistortStart - (offsetDistortUniformIncrF * i)) - (offsetDistortIncrF * i));
        
        scaleCurF *= scaleRootMultiplierF;
        
        fragA += texture(videoTexture, offsetCoordsA);
        fragB += texture(videoTexture, offsetCoordsB);
    }
    
    fragA *= vec4(iterationMultF);
    fragB *= vec4(iterationMultF);
    
    //fragA = vec4(scaleRootMultiplierF);

    // here we only care about the aberrated channels and their alpha's (2 + 2 channels, that's all we can fit in the RGBA). the unchanged channel will be re-sampled in the postProc shader.
    // red + alpha, and blue + alpha, the default behaviour
    
    if(mode == 1)
    {
        FragColour = vec4(fragA.r, fragA.a, fragB.b, fragB.a); // red blue
    }
    else if(mode == 2)
    {
        FragColour = vec4(fragA.r, fragA.a, fragB.g, fragB.a); // red green
    }
    else
    {
        FragColour = vec4(fragA.g, fragA.a, fragB.b, fragB.a); // green blue
    }
}
