#version 330
uniform sampler2D videoTexture;
out vec4 FragColour;
in vec2 out_uvs;

uniform int stepsA;
uniform float strideA;
uniform int stepsB;
uniform float strideB;

float getWeight(float x)
{
    //return exp(-0.5*(x*x)/(0.5)); // magic number is sigma squared
    return exp(-0.5*(x*x)/0.06);
}
           
// blur H
void main(void)
{
    // blurA
    int STEPS = stepsA;
    vec4 totalA = vec4(0.0);
    vec4 totalWeight = vec4(0.0);
    
    for(int i = -STEPS; i <= STEPS; i++)
    {
        vec2 texCoord = out_uvs.xy + vec2(0.0, float(i) * strideA);
        float weight = getWeight(float(i) / float(STEPS));
        totalA += vec4(weight) * texture(videoTexture, texCoord.xy);
        totalWeight += vec4(weight);
    }
    
    totalA /= totalWeight;
    
    // blurB
    STEPS = stepsB;
    vec4 totalB = vec4(0.0);
    totalWeight = vec4(0.0);
    
    for(int i = -STEPS; i <= STEPS; i++)
    {
        vec2 texCoord = out_uvs.xy + vec2(0.0, float(i) * strideB);
        float weight = getWeight(float(i) / float(STEPS));
        totalB += vec4(weight) * texture(videoTexture, texCoord.xy);
        totalWeight += vec4(weight);
    }
    
    totalB /= totalWeight;

    FragColour.r = totalA.r; // red in red
    FragColour.g = totalA.g; // red's alpha in green
    FragColour.b = totalB.b; // blue in blue
    FragColour.a = totalB.a; // blue's alpha in alpha
    
    // debug: this shader does nothing
    //FragColour = texture(videoTexture, out_uvs);
}
